package apps.yesim.security;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import lombok.extern.java.Log;

import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.AuthenticationInfo;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.authc.SimpleAccount;
import org.apache.shiro.authc.UsernamePasswordToken;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.apache.shiro.realm.AuthorizingRealm;
import org.apache.shiro.subject.PrincipalCollection;

import apps.yesim.domain.entity.Person;
import apps.yesim.util.jpa.EntityManagerBuilder;

/**
 * 
 * @author Gabriel Cardelli
 *
 */

@javax.inject.Singleton
@Log
public class JpaRealm extends AuthorizingRealm {

	private EntityManager em;
	
	protected SimpleAccount getAccount(String email,char[] password) {
		
		em = EntityManagerBuilder.getEntityManager();
		
		Query query = em.createNamedQuery("getPersonByEmailAndPassword");
		query.setParameter(1, email);
		query.setParameter(2,new String(password));
		
		log.info("Email: " + email);
		log.info("Pass: " + new String(password));
		
		Person person = (Person) query.getSingleResult();
		
		SimpleAccount account = new SimpleAccount(person.getEmail(),person.getPassword(), getName());
		account.setRoles(person.getRolesName());
		
		return account;
		
	}

	@Override
	protected AuthorizationInfo doGetAuthorizationInfo(PrincipalCollection principalCollection) {
		
		String email = (String) getAvailablePrincipal(principalCollection);
		
		Query query = em.createNamedQuery("getRolesNameByPersonEmail");
		
		query.setParameter(1, email);
		
		List<String> result = (List<String>) query.getResultList();
		
		Set<String> rolesName = new HashSet<String>(result);
		
		
		SimpleAuthorizationInfo simpleAuthorizationInfo = new SimpleAuthorizationInfo();
		
		simpleAuthorizationInfo.setRoles(rolesName);
		
		return simpleAuthorizationInfo;
		

	}

	@Override
	protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken token) throws AuthenticationException {
		UsernamePasswordToken upToken = (UsernamePasswordToken) token;
		return getAccount(upToken.getUsername(),upToken.getPassword());
	}

}
